package com.supwisdom.dlpay

import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import org.jose4j.jwt.consumer.InvalidJwtException
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.http.HttpStatus
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter
import org.springframework.security.web.authentication.rememberme.JdbcTokenRepositoryImpl
import org.springframework.security.web.util.matcher.AntPathRequestMatcher
import org.springframework.web.filter.OncePerRequestFilter
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import javax.sql.DataSource


class ApiJwtAuthenticationFilter(jwt: JwtTokenUtil) : OncePerRequestFilter() {
    private val jwtUtil = jwt

    override fun doFilterInternal(request: HttpServletRequest, response: HttpServletResponse, filterChain: FilterChain) {
        request.getHeader(jwtUtil.header)?.let { jwt ->
            try {
                val claims = jwtUtil.verifyToken(jwt)
                val auth = UsernamePasswordAuthenticationToken(claims["uid"], null,
                        claims["authorities"]?.map { SimpleGrantedAuthority(it as String) })
                SecurityContextHolder.getContext().authentication = auth
            } catch (e: InvalidJwtException) {
                SecurityContextHolder.clearContext();
                response.sendError(HttpStatus.BAD_REQUEST.value(), e.message)
            }
        }
        filterChain.doFilter(request, response)
    }
}

@EnableWebSecurity
class WebSecurityConfig {

//    @Bean
//    fun daoProvider(detailsService: UserDetailsService): DaoAuthenticationProvider {
//        return DaoAuthenticationProvider().also {
//            it.setUserDetailsService(detailsService)
//        }
//    }
//
//    @Bean
//    fun providerManager(daoProvider: DaoAuthenticationProvider): ProviderManager {
//        return ProviderManager(listOf(daoProvider))
//    }


    companion object {
        @Configuration
        @Order(1)
        class ApiWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {

            @Autowired
            lateinit var jwtConfig: JwtConfig


            override fun configure(http: HttpSecurity) {
                // 设置 API 访问权限管理
                http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                        .and()
                        .addFilterAfter(ApiJwtAuthenticationFilter(JwtTokenUtil(jwtConfig)),
                                UsernamePasswordAuthenticationFilter::class.java)
                        .authorizeRequests()
                        .antMatchers("/api/auth/**").permitAll()
                        .antMatchers("/api/common/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/consume/**").hasRole("THIRD_CONSUME")
                        .antMatchers("/api/deposit/**").hasRole("THIRD_DEPOSIT")
                        .antMatchers("/api/user/**").hasRole("THIRD_ADMIN")
                        .antMatchers("/api/shop/**").hasRole("THIRD_SHOP")
                        .and()
                        .csrf().ignoringAntMatchers("/api/**")
            }
        }

        @Configuration
        class MvcWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {
            @Autowired
            lateinit var dataSource: DataSource
//            @Autowired
//            lateinit var validateCodeSecurityConfig: ValidateCodeSecurityConfig
//            @Autowired
//            lateinit var userDetailsService: OperatorDetailService
//            @Autowired
//            lateinit var myAuthenticationFailureHandler: AuthenticationFailureHandler
//            @Autowired
//            lateinit var myAuthenticationSuccessHandler: AuthenticationSuccessHandler
//            @Autowired
//            lateinit var myInvalidSessionStrategy: InvalidSessionStrategy


            @Bean
            fun jdbcTokenImplement(): JdbcTokenRepositoryImpl {
                return JdbcTokenRepositoryImpl().also {
                    it.setDataSource(dataSource)
                }
            }

            override fun configure(http: HttpSecurity) {
                // 设置 Web MVC 应用权限
                http.csrf()
                        .and()
                        .authorizeRequests()
                        .antMatchers("/login", "/login/form").permitAll()
                        .antMatchers("/static/**").permitAll()
                        .antMatchers("/code/image").permitAll()
                        .anyRequest().authenticated()
                        .and()
                        .sessionManagement()
                        .sessionCreationPolicy(SessionCreationPolicy.ALWAYS)
                        .and()
                        .formLogin()
                        .loginPage("/login")
                        .loginProcessingUrl("/login/form")
                        .and()
                        .logout()
                        .logoutRequestMatcher(AntPathRequestMatcher("/logout"))
                        .logoutSuccessUrl("/login")
                        .deleteCookies("JSESSIONID")
                        .invalidateHttpSession(true)
                // 设置 Web MVC 应用权限
//                http.apply(validateCodeSecurityConfig)
//                        .and()
//                        .formLogin()
//                        .loginPage("/login")
//                        .loginProcessingUrl("/login/form")
//                        .successHandler(zyAuthenticationSuccessHandler)
//                        .failureHandler(zyAuthenticationFailureHandler)
//                        .and()
//                        .logout()
//                        .logoutRequestMatcher(AntPathRequestMatcher("/logout"))
//                        .logoutSuccessUrl("/login")
//                        .deleteCookies("JSESSIONID")
//                        .invalidateHttpSession(true)
//                        .and()
//                        .userDetailsService(userDetailsService)
//                        .authorizeRequests()
//                        .antMatchers("/login").permitAll()
//                        .antMatchers("/static/**").permitAll()
//                        .antMatchers("/code/image").permitAll()
//                        .anyRequest().authenticated()
//                        .and()
//                        .sessionManagement()
//                        .invalidSessionStrategy(myInvalidSessionStrategy)
//                        .maximumSessions(1)
//                        .sessionRegistry(SessionRegistryImpl())
//                        .maxSessionsPreventsLogin(true)
//                        .and()
//                        .and()
//                        .headers().frameOptions().disable()
            }
        }
    }
}
